/*
 * Copyright (c) 2001-2004 Swedish Institute of Computer Science.
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without modification,
 * are permitted provided that the following conditions are met:
 *
 * 1. Redistributions of source code must retain the above copyright notice,
 *    this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright notice,
 *    this list of conditions and the following disclaimer in the documentation
 *    and/or other materials provided with the distribution.
 * 3. The name of the author may not be used to endorse or promote products
 *    derived from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT
 * SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT
 * OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING
 * IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY
 * OF SUCH DAMAGE.
 *
 * This file is part of the lwIP TCP/IP stack.
 *
 * Author: Huawei Technologies
 *
 */

#ifndef LWIP_ARCH_SYS_ARCH_H
#define LWIP_ARCH_SYS_ARCH_H

#include <rte_cycles.h>
#include <rte_debug.h>

#define SYS_MBOX_NULL NULL
#define SYS_SEM_NULL  NULL
typedef uint32_t sys_prot_t;

struct sys_sem {
    volatile unsigned int c;
    int (*wait_fn)(void);
};

#define MBOX_NAME_LEN 64
struct sys_mbox {
    struct rte_ring *ring;
    char name[MBOX_NAME_LEN];
    int size;
    int socket_id;
    unsigned flags;
    int (*wait_fn)(void);
};

typedef struct sys_sem *sys_sem_t;
#define sys_sem_valid(sem)             (((sem) != NULL) && (*(sem) != NULL))
#define sys_sem_valid_val(sem)         ((sem) != NULL)
#define sys_sem_set_invalid(sem)       do { if ((sem) != NULL) { *(sem) = NULL; }} while(0)
#define sys_sem_set_invalid_val(sem)   do { (sem) = NULL; } while(0)

struct sys_mutex;
typedef struct sys_mutex *sys_mutex_t;
#define sys_mutex_valid(mutex)         sys_sem_valid(mutex)
#define sys_mutex_set_invalid(mutex)   sys_sem_set_invalid(mutex)

typedef struct sys_mbox *sys_mbox_t;
#define sys_mbox_valid(mbox)           sys_sem_valid(mbox)
#define sys_mbox_valid_val(mbox)       sys_sem_valid_val(mbox)
#define sys_mbox_set_invalid(mbox)     sys_sem_set_invalid(mbox)
#define sys_mbox_set_invalid_val(mbox) sys_sem_set_invalid_val(mbox)
int sys_mbox_empty(struct sys_mbox *);

struct sys_thread;
typedef struct sys_thread *sys_thread_t;

#if USE_LIBOS
extern int eth_dev_poll(void);
#include <rte_ring.h>

/* 
    gazelle custom rte ring interface
    lightweight ring no atomic.
    only surpport in single thread.
 */
static __rte_always_inline uint32_t gazelle_st_ring_enqueue_busrt(struct rte_ring *r, void **obj_table, uint32_t n)
{
    uint32_t prod = r->prod.tail;
    uint32_t cons = r->cons.tail;
    uint32_t free_entries = r->capacity + cons - prod;

    if (n > free_entries) {
        return 0;
    }

    ENQUEUE_PTRS(r, &r[1], prod, obj_table, n, void *);

    r->prod.tail = prod + n;

    return n;
}

static __rte_always_inline uint32_t gazelle_st_ring_dequeue_burst(struct rte_ring *r, void **obj_table, uint32_t n)
{
    uint32_t cons = r->cons.tail;
    uint32_t prod = r->prod.tail;
    uint32_t entries = prod - cons;

    if (n > entries) {
        n = entries;
    }

    if (n == 0) {
        return 0;
    }

    DEQUEUE_PTRS(r, &r[1], cons, obj_table, n, void *);

    r->cons.tail = cons + n;

    return n;
}
#endif

void sys_calibrate_tsc(void);
uint32_t sys_now(void);
__attribute__((always_inline)) inline int update_timeout(int timeout, uint32_t poll_ts)
{
    uint32_t used_ms = sys_now() - poll_ts;
    if (timeout > 0 && used_ms < timeout) {
        return timeout;
    } else {
        return 0;
    }
}

#endif /* LWIP_ARCH_SYS_ARCH_H */
